package ca.canadapost.cpcdp.shipping;

import ca.canadapost.cpcdp.shipping.stub.shipment.*;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.cxf.interceptor.LoggingInInterceptor;
import org.apache.cxf.interceptor.LoggingOutInterceptor;
import ca.canadapost.cpcdp.shipping.util.ClientUtil;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;
import java.util.Map;

/**
 * Sample code for the GetShipmentPrice Canada Post service.
 * <p>
 * The GetShipmentPrice service is used to retrieve the shipment pricing
 * in XML format for a shipment created by a prior "create shipment" call.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 */
public class GetShipmentPriceClient {
	// Main class for local testing only
    public static void main(String[] args) {
        try{
			// Your username and password are imported from the following file: CPCWS_SOAP_Shipping_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");
            String mailedBy = userProperties.get("mailedBy");

            ShipmentService service = new ShipmentService();
            ShipmentPortType port = service.getShipmentPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/shipment/v8");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //build request
            GetShipmentPriceRequest request = new GetShipmentPriceRequest();
            request.setLocale("FR");
            request.setMailedBy(mailedBy);
            request.setShipmentId("522121392859903208");

            //execute request
            GetShipmentPriceResponse response = port.getShipmentPrice(request);

            //retrieve values from response
            if(response.getMessages() == null){
                ShipmentPriceType shipmentPriceType = response.getShipmentPrice();
                System.out.println("Service Code: " + shipmentPriceType.getServiceCode());
                System.out.println("Due amount: " + shipmentPriceType.getDueAmount());
            } else {
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
