package ca.canadapost.cpcdp.shipping;

import ca.canadapost.cpcdp.shipping.stub.shipment.*;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import ca.canadapost.cpcdp.shipping.util.ClientUtil;

import javax.xml.bind.JAXBElement;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;

import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Map;

/**
 * Sample code for the GetShipmentPublicKeyInfo Canada Post service.
 * <p>
 * The GetShipmentPublicKeyInfo service is used to retrieve the public key info in XML
 * format for a shipment created by a prior "create shipment" call.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 */
public class GetShipmentPublicKeyInfoClient {
	// Main class for local testing only
    public static void main(String[] args) {
        try{
			// Your username and password are imported from the following file: CPCWS_SOAP_Shipping_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");
            String mailedBy = userProperties.get("mailedBy");

            ShipmentService service = new ShipmentService();
            ShipmentPortType port = service.getShipmentPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/shipment/v8");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //build request
            GetPublicKeyInfoRequest request = new GetPublicKeyInfoRequest();
            request.setLocale("FR");
            request.setMailedBy(mailedBy);
            request.setShipmentId("362741689768098726");

            //execute request
            GetPublicKeyInfoResponse response = port.getShipmentPublicKeyInfo(request);

            //retrieve values from response
            if(response.getMessages() == null){
                PublicKeyInfoType publicKeyInfoType = response.getShipmentPublicKeyInfo();
                System.out.println("Expiry Date: " + publicKeyInfoType.getExpiryDate());
                System.out.println("Url: " + publicKeyInfoType.getUrl());
                if(publicKeyInfoType.getQrCode() != null) {
            		//write QR code to file
            		Path pathToQRCodeImage = Paths.get("qr-code.jpeg");
            		Files.write(pathToQRCodeImage, publicKeyInfoType.getQrCode());
            		System.out.println("QR code image created at: " + pathToQRCodeImage.toAbsolutePath());
            	}
            } else {
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
