package ca.canadapost.cpcdp.shipping;

import ca.canadapost.cpcdp.shipping.stub.*;
import ca.canadapost.cpcdp.shipping.stub.shipment.*;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.cxf.interceptor.LoggingOutInterceptor;
import ca.canadapost.cpcdp.shipping.util.ClientUtil;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;
import java.util.Map;

/**
 * Sample code for the VoidShipment Canada Post service.
 * <p>
 * The VoidShipment service is used to indicate to Canada Post that a printed label
 * has been spoiled or cancelled and that it will not be used and should not be
 * transmitted or billed.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 */
public class VoidShipmentClient {
	// Main class for local testing only
    public static void main(String[] args) {
        try{
            // Your username and password are imported from the following file: CPCWS_SOAP_Shipping_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");
            String mailedBy = userProperties.get("mailedBy");

            ShipmentService service = new ShipmentService();
            ShipmentPortType port = service.getShipmentPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/shipment/v8");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //log request/response
            //client.getInInterceptors().add(new LoggingInInterceptor());
            //client.getOutInterceptors().add(new LoggingOutInterceptor());

            //build request
            VoidShipmentRequest request = new VoidShipmentRequest();
            request.setMailedBy(mailedBy);
            request.setLocale("FR");
            request.setShipmentId("826131401815714546");

            //execute request
            VoidShipmentResponse response = port.voidShipment(request);

            //retrieve values from response
            if(Boolean.TRUE.equals(response.isVoidShipmentSuccess())){
                System.out.println("Shipment successfully deleted.");
            } else {
                //assume error
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }

        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
